#include "Defs.h"
#include "HwDefs.h"
#include <avr\io.h>
#include <avr\interrupt.h>

BYTE FUartSendBuf[64];
BYTE FUartRecvBuf[256];

BYTE FUartRecvSize,
     FUartPackSize;
BYTE FUartSendBegin,
     FUartSendEnd;

void SwitchTask ( void );
void Maint ( void ); // Must be declared externally in the application

void BiosInit ( void )
{
 FUartRecvSize=0;
 FUartPackSize=0;
 FUartSendBegin=0;
 FUartSendEnd=0;

 /* UART */
 UBRR0H=0;
 UBRR0L=28;  /* 13560/(8*(X+1))=BaudRate */
 UCSR0A=0x02;
 UCSR0B=0x98;
 UCSR0C=0x06;

 /* ASSR */
 ASSR=0;

 /* Timer 1 */
 TCCR1A=0x00;
 OCR1B=0;
 OCR1A=13560;
 TCCR1B=0x09;
 TCNT1=0;
 TIMSK=0x10;

 /* PORT A */
 PORTA=0x00;
 DDRA=PortADirection;

 PORTF=0xFF;
 DDRF=0xFF;
 PORTC=0x00;
 DDRC=0xFF;

 SclkLow;
 ModeComm;
}

ISR(USART0_RX_vect)
{
 BYTE BData;

 BData=UDR0;
 if (FUartRecvSize<255)
  {
   FUartRecvBuf[FUartRecvSize]=BData; FUartRecvSize++;
   if ((BData==13)&&(FUartPackSize==0)) FUartPackSize=FUartRecvSize;
  }
}

ISR(USART0_UDRE_vect)
{
 if (FUartSendBegin==FUartSendEnd) UCSR0B&=0xDF;
 else
  {
   UDR0=FUartSendBuf[FUartSendBegin];
   FUartSendBegin=(FUartSendBegin+1)&63;
  }
}

void UartSendByte ( BYTE AData )
{
 BYTE BData;
 BYTE BIndex;

 BIndex=(FUartSendEnd+1)&63;
 while (BIndex==FUartSendBegin) SwitchTask();
 FUartSendBuf[FUartSendEnd]=AData;
 FUartSendEnd=BIndex;

 if ((UCSR0B&0x20)==0)
  {
   while ((UCSR0A&0x20)==0) SwitchTask();
   BData=FUartSendBuf[FUartSendBegin]; FUartSendBegin=(FUartSendBegin+1)&63;
   UCSR0B|=0x20;
   UDR0=BData;
  }
}

ISR(TIMER1_COMPA_vect)
{
 TIFR=0x10;
}

void SwitchTask ( void )
{
 Maint();
}


