#include "Defs.h"
#include "HwDefs.h"
#include <avr\io.h>
#include <avr\interrupt.h>

BYTE FUartSendBuf[64];
BYTE FUartRecvBuf[256];

BYTE FUartRecvSize,
     FUartPackSize;
BYTE FUartSendBegin,
     FUartSendEnd;

void SwitchTask ( void );
void Maint ( void ); // Must be declared externally in the application

void BiosInit ( void )
{
 FUartRecvSize=0;
 FUartPackSize=0;
 FUartSendBegin=0;
 FUartSendEnd=0;

 /* UART */
 UBRRH=0;
 UBRRL=28;  /* 13560/(8*(X+1))=BaudRate */
 UCSRA=0x02;
 UCSRB=0x98;
 UCSRC=0x86;//Set URSEL to one to write into the UCSRD register

 /* ASSR */
 ASSR=0x00;	//Clear AS2 bit to clock Timer2 from crystal oscillator

 /* Timer 1 */
 TCCR1A=0x00;
 OCR1B=0;
 OCR1A=13560;
 TCCR1B=0x09;
 TCNT1=0;
 TIMSK=0x10;

 /* PORT B */
 PORTB=0x02;
 DDRB=PortBDirection;

 /* PORT D */
 PORTD=0x00;	
 DDRD=PortDDirection;

 /* PORT C */
 PORTC=0x00;
 DDRC=PortCDirection;

 SclkLow;
 ModeComm;
}

ISR(USART_RXC_vect)
{
 BYTE BData;

 BData=UDR;
 if (FUartRecvSize<255)
  {
   FUartRecvBuf[FUartRecvSize]=BData; FUartRecvSize++;
   if ((BData==13)&&(FUartPackSize==0)) FUartPackSize=FUartRecvSize;
  }
}

ISR(USART_UDRE_vect)
{
 if (FUartSendBegin==FUartSendEnd) UCSRB&=0xDF;
 else
  {
   UDR=FUartSendBuf[FUartSendBegin];
   FUartSendBegin=(FUartSendBegin+1)&63;
  }
}

void UartSendByte (BYTE AData)
{
 BYTE BData;
 BYTE BIndex;

 BIndex=(FUartSendEnd+1)&63;
 while (BIndex==FUartSendBegin) SwitchTask();
 FUartSendBuf[FUartSendEnd]=AData;
 FUartSendEnd=BIndex;

 if ((UCSRB&0x20)==0)
  {
   while ((UCSRA&0x20)==0) SwitchTask();
   BData=FUartSendBuf[FUartSendBegin]; FUartSendBegin=(FUartSendBegin+1)&63;
   UCSRB|=0x20;
   UDR=BData;
  }
}

ISR(TIMER1_COMPA_vect)
{
 TIFR=0x10;
}

void SwitchTask ( void )
{
 Maint();
}


