#include "Defs.h"
#include "HwDefs.h"
#include "IsoX.h"
#include <avr\io.h>
#include "ErrorCodes.h"
#include "Crt.h"
#include "Mlx90121.h"

#define CIso14BRecvDelay        63
#define CIso14BPollDataStable   20
#define CIso14B_TR1Wait         200
#define CIso14B_SofAWait        200
#define CIso14B_SofBWait        50

void Iso14BSendByte ( BYTE AData )
{
 BYTE BBitIndex;

 // Start bit
 MlxWaitDSYN();
 DataLow;
 SclkHigh; SclkHigh; SclkHigh; SclkHigh; SclkLow;

 // Data bits
 for (BBitIndex=0; BBitIndex<8; BBitIndex++)
  {
   if (MlxWaitDSYN()==FALSE) break;

   if (AData&0x01) DataHigh;
   else DataLow;
   SclkHigh;
   SclkHigh;
   SclkHigh;
   AData>>=1;
   SclkLow;
  }

 // Stop bit
 MlxWaitDSYN();
 DataHigh;
 SclkHigh; SclkHigh; SclkHigh; SclkHigh; SclkLow;

}

BYTE Iso14BSend ( BYTE *ABuf, BYTE ASize, BYTE AAutoCrc )
{
 BYTE BResult;
 WORD BCrc;
 BYTE BIndex;

 BCrc=0; // Just for compiler

 if (AAutoCrc) BCrc=IsoGetCrc(ABuf,ASize,CIso14BCrcPoly)^0xFFFF;
 MlxSendStart(0x0B,127,0);

 for (BIndex=0; BIndex<9; BIndex++) MlxSendNextSym1P(0);
 for (BIndex=0; BIndex<2; BIndex++) MlxSendNextSym1P(1);
 for (BIndex=0; BIndex<ASize; BIndex++) Iso14BSendByte(ABuf[BIndex]);
 if (AAutoCrc)
  {
   Iso14BSendByte(BCrc&0xFF);
   Iso14BSendByte(BCrc>>8);
  }
 for (BIndex=0; BIndex<10; BIndex++) MlxSendNextSym1P(0);
 BResult=MlxSendStop(0x0A,CIso14BRecvDelay);

 return BResult;
}

BYTE Iso14BRecv ( BYTE *ABuf, BYTE ABufSize, BYTE *ABytesReceived )
{
 BYTE BResult;
 BYTE BIntSave;
 BYTE BData,
      BMask,
      BMaskX;
 BYTE BPollData;
 BYTE BByteCount,
      BBitIndex;

 BResult=ENoTag;
 BByteCount=0;
 TrbRead; ModeComm;

 BIntSave=IntDisable();
 while ((TimerTest)==0);

 do
  {
   // Polling data stability on DOUT
   BPollData=0; BData=0;
   TimerSet(0x0C,120);
   while (TimerTest==0)
    {
     BData=GetData;
     for (BPollData=0; BPollData<CIso14BPollDataStable; BPollData++)
      {
       if (TimerTest) break;
       if (BData!=GetData) break;
      }
     if (BPollData>=CIso14BPollDataStable) break;
    }
   if (BPollData<CIso14BPollDataStable) break;

   // Data in DOUT is stable
   // TR1
   TimerSet(0x0A,CIso14B_TR1Wait);
   BMask=BData^0x01;
   if (BMask==0) BMaskX=0x00;
   else BMaskX=0xFF;
   while (TimerTest==0) if (BMask==GetData) break;
   if (TimerTest) { BResult=EInvalidSof; break; }
   PORTC=0x01;

   // SOF A (10-11 bits)
   TimerSet(0x0A,CIso14B_SofAWait);
   while (TimerTest==0) if (BMask!=GetData) break;
   if (TimerTest) { BResult=EInvalidSof; break; }
   PORTC=0x03;

   // SOF B (2-3 bits)
   TimerSet(0x0A,CIso14B_SofBWait);
   while (TimerTest==0) if (BMask==GetData) { SclkHigh; break; }
   if (TimerTest) { BResult=EInvalidSof; break; }
   PORTC=0x02;

   TimerSet(0x0A,128); // bit and EGT wait
   BByteCount=0;
   do
    {
     if (MlxWaitDSYN()==FALSE) { BResult=ENoDSYN; break; }
     if (BMask!=GetData) { BResult=EFramingError; break; }
     BData=0;
     for (BBitIndex=0; BBitIndex<8; BBitIndex++)
      {
       if (MlxWaitDSYN()==FALSE) break;
       BData=(BData>>1)|(GetData<<7);
      }
     SclkLow;
     BData^=BMaskX;
     SclkLow; SclkLow;
     if (GetData==BMask)
      {
       if (BData==0x00) BResult=EFrameRecvOK; // EOF received
       else BResult=EFramingError;
       break;
      }
     if (BByteCount>=ABufSize) { BResult=EBufOverflow; break; }
     *ABuf=BData; ABuf++; BByteCount++;
     // EGT resynchro
     PORTC=0x03;
     TimerReset;
     while (TimerTest==0) if (GetData==BMask) { SclkHigh; break; }
     if (TimerTest) { BResult=EEgtError; break; }
     PORTC=0x02;
    } while(1);

  } while(0);

 *ABytesReceived=BByteCount;

 SclkLow;
 IntRestore(BIntSave);

 PORTC=0x00;
 return BResult;
}

