#include "Defs.h"
#include "HwDefs.h"
#include "IsoX.h"
#include <avr\io.h>
#include "ErrorCodes.h"
#include "Crt.h"
#include "Mlx90121.h"

#define CIso15_A  0
#define CIso15_B  1
#define CIso15_C  2
#define CIso15_D  3
#define CIso15Sof 4
#define CIso15Eof 5

#define CIso15RecvDelay 44

void Iso15SendByte ( BYTE AData )
{
 BYTE BDualIndex;

 for (BDualIndex=0; BDualIndex<4; BDualIndex++)
  {
   if (MlxWaitDSYN()==FALSE) break;

   if (AData&0x02) DataHigh;
   else DataLow;
   SclkHigh;
   SclkHigh;
   SclkHigh;
   SclkHigh;
   SclkLow;

   if (AData&0x01) DataHigh;
   else DataLow;
   SclkHigh;
   SclkHigh;
   SclkHigh;
   AData>>=2;
   SclkLow;
  }
}

BYTE Iso15Send ( BYTE *ABuf, BYTE ASize, BYTE AAutoCrc )
{
 BYTE BResult;
 WORD BCrc;
 BYTE BIndex;
 BYTE BIntSave;

 BCrc=0; // Just for compiler
 BIntSave=0;

 BResult=ENoError;
 if (ASize==0) // Send only EOF
  {
   BIntSave=IntDisable();
   MlxSendStart(0x0B,127,CIso15Eof);
   BResult=MlxSendStop(0x0B,CIso15RecvDelay);
  }
 else // Send a complete frame
  {
   if (AAutoCrc) BCrc=IsoGetCrc(ABuf,ASize,CIso15CrcPoly)^0xFFFF;
   BIntSave=IntDisable();
   MlxSendStart(0x0B,127,CIso15Sof);
   for (BIndex=0; BIndex<ASize; BIndex++) Iso15SendByte(ABuf[BIndex]);
   if (AAutoCrc)
    {
     Iso15SendByte(BCrc&0xFF);
     Iso15SendByte(BCrc>>8);
    }
   MlxSendNextSym3P(CIso15Eof);
   BResult=MlxSendStop(0x0B,CIso15RecvDelay);
  }

 if (BIntSave) IntRestore(BIntSave);

 return BResult;
}

BYTE Iso15Recv ( BYTE *ABuf, BYTE ABufSize, BYTE *ABytesReceived, BYTE *AFlags )
{
 BYTE BResult;
 BYTE BIntSave;
 BYTE BBitIndex;
 BYTE BDataA, BDataB;
 BYTE BByteCount;
 BYTE BFlags;

 BResult=ENotStarted;
 BFlags=0; BByteCount=0;
 TrbRead; ModeComm;

 while ((TimerTest)==0);

 BIntSave=IntDisable();

 do
  {
   TimerSet(0x0A,80); while (TimerTest==0);

   if (MlxSyncStartASK(0x0A,130)==FALSE) { BResult=EFrameWaitTOut; break; }

   TimerSet(0x0A,63); // Bit wait time

   BDataA=0;
   for (BBitIndex=0; BBitIndex<5; BBitIndex++)
    {
    if (MlxWaitDSYN()==FALSE) { BResult=ENoDSYN; break; }
     BDataA=(BDataA<<1)|GetData;
    }
   if (BBitIndex!=5) break;
   if (BDataA!=0x1D) { BResult=EInvalidSof; break; }

   BByteCount=0;
   do
    {
     BDataA=0; BDataB=0;
     for (BBitIndex=0; BBitIndex<8; BBitIndex++)
      {
       if (MlxWaitDSYN()==FALSE) { BResult=ENoDSYN; break; }
       BDataA=(BDataA>>1)|(GetData<<7);

       if (MlxWaitDSYN()==FALSE) { BResult=ENoDSYN; break; }
       BDataB=(BDataB>>1)|(GetData<<7);

       if ((BBitIndex==3)&&(BDataA==0x70)&&(BDataB==0x20)) { BResult=EFrameRecvOK; break; }
      }
     if (BResult!=ENotStarted) break;
     if (BByteCount>=ABufSize) { BResult=EBufOverflow; break; }
     *ABuf=BDataB; ABuf++; BByteCount++;
     if ((BDataA^BDataB)!=0xFF) BFlags|=0x01; // Collision detected
    } while(1);

  } while(0);

 *ABytesReceived=BByteCount;
 *AFlags=BFlags;
 
 MlxSyncStop();
 IntRestore(BIntSave);

 return BResult;
}

