#include "Defs.h"
#include "Crt.h"
#include "IsoX.h"
#include "Iso15.h"
#include "ErrorCodes.h"

#define CIso15InvNoTag 0
#define CIso15InvFound 1
#define CIso15InvColl  2
#define CIso15InvError 3

//#define CIso15InvDebug

#ifdef CIso15InvDebug
#include <avr\pgmspace.h>
#include "Comm.h"
#endif

BYTE FIso15InvMode;
BYTE FIso15InvLevelIndex;
BYTE FIso15InvSlotIndex;
WORD FIso15Coll[16];

void Iso15InvStart ( BYTE AMode )
{
 FIso15InvMode=AMode;
 FIso15InvLevelIndex=0;
 FIso15InvSlotIndex=0;
 FIso15Coll[0]=0;
}

BYTE Iso15InvGetBitIndex ( WORD AData )
{
 BYTE BIndex;
 WORD BCmp;

 BCmp=0x0001;
 for (BIndex=0; BIndex<16; BIndex++)
  {
   if (AData&BCmp) break;
   BCmp<<=1;
  }
  
 return BIndex;
}

BYTE Iso15InvSendRecv ( BYTE *ABuffer )
{
 BYTE BWrIndex;
 BYTE BLevelIndex;
 BYTE BMask;
 BYTE BResult;
 BYTE BBytesReceived;
 BYTE BFlags;
 WORD BCrc;

 if (FIso15InvSlotIndex!=0)
  {
   #ifdef CIso15InvDebug
   CommSendStrC13(PSTR("   EOF ->"));
   #endif
   Iso15Send(0,0,FALSE);
  }
 else
  {
   ABuffer[0]=FIso15InvMode;
   ABuffer[1]=0x01;
   ABuffer[2]=FIso15InvLevelIndex<<2;
   BWrIndex=3;
   for (BLevelIndex=0; BLevelIndex<FIso15InvLevelIndex; BLevelIndex++)
    {
     BMask=Iso15InvGetBitIndex(FIso15Coll[BLevelIndex]);
     if (BLevelIndex&0x01) ABuffer[BWrIndex-1]|=(BMask<<4);
     else { ABuffer[BWrIndex]=BMask; BWrIndex++; }
    }
   #ifdef CIso15InvDebug
   CommSendStrC(PSTR("   Command ")); CommSendBufHex(ABuffer,BWrIndex); CommSendStrC13(PSTR(" ->"));
   #endif

   Iso15Send(ABuffer,BWrIndex,TRUE);
  }

 BResult=Iso15Recv(ABuffer,12,&BBytesReceived,&BFlags);
 #ifdef CIso15InvDebug
 CommSendStrC(PSTR("     <- ")); CommSendBufHex(ABuffer,BBytesReceived);
 CommSendStrC(PSTR(" Result=")); CommSendDec(BResult);
 #endif

 switch (BResult)
  {
   case EFrameRecvOK:
     if (BBytesReceived!=12) { BResult=CIso15InvColl; break; }
     if (BFlags&0x01) { BResult=CIso15InvColl; break; }
     BCrc=IsoGetCrc(ABuffer,BBytesReceived,CIso15CrcPoly);
     if (BCrc!=0xF0B8) { BResult=CIso15InvColl; break; }
     BResult=CIso15InvFound;
     break;

   case ENoDSYN:
     BResult=CIso15InvError;
     break;

   case EFrameWaitTOut:
   case EInvalidSof:
     BResult=CIso15InvNoTag;
     break;

   case EBufOverflow:
     BResult=CIso15InvColl;
     break;

   default:
     BResult=CIso15InvError;
  }

 #ifdef CIso15InvDebug
 if (BResult==CIso15InvColl) CommSendStrC(PSTR(" Coll"));
 CommSendStrC13(PSTR(""));
 #endif

 return BResult;
}

BYTE Iso15InvGetNext ( BYTE *ABuffer )
{
 BYTE BResult;
 BYTE BResultA;
 BYTE BCollIndex;

 BResult=ENoTag;
 do
  {
   if (FIso15InvSlotIndex<16)
    {
     BResultA=Iso15InvSendRecv(ABuffer);
     if (BResultA==CIso15InvNoTag) FIso15InvSlotIndex++;
     else if (BResultA==CIso15InvFound) { FIso15InvSlotIndex++; BResult=EFrameRecvOK; break; }
     else if (BResultA==CIso15InvColl) { FIso15Coll[FIso15InvLevelIndex]|=(1<<FIso15InvSlotIndex); FIso15InvSlotIndex++; }
     else /* CIso15Error */ { BResult=ECommError; break; }
    }
   else if (FIso15Coll[FIso15InvLevelIndex])
    {
     if (FIso15InvLevelIndex<15)
      {
       FIso15InvLevelIndex++;
       FIso15Coll[FIso15InvLevelIndex]=0;
       FIso15InvSlotIndex=0;
      }
     else // There should be no collision in the last level; Just reset flag, we can do nothing about this
      {
       BCollIndex=Iso15InvGetBitIndex(FIso15Coll[FIso15InvLevelIndex]);
       FIso15Coll[FIso15InvLevelIndex]&=((1<<BCollIndex)^0xFFFF);
      }
    }
   else
    {
     if (FIso15InvLevelIndex==0) break; // No more tags
     FIso15InvLevelIndex--;
     BCollIndex=Iso15InvGetBitIndex(FIso15Coll[FIso15InvLevelIndex]);
     FIso15Coll[FIso15InvLevelIndex]&=((1<<BCollIndex)^0xFFFF);
     FIso15InvSlotIndex=16; // In order to block re-verification of previous level (! This line is not necessary and can be optimized)
    }
  } while(1);

 return BResult;
}


